@@ -17,7 +17,7 @@ TARGET="$(uname -m)"
 TARGET_CFLAGS=''
 QUIET='VERBOSE=1'
 WITH_JEMALLOC=''
-DEBUG_EXTNAME=''
+DEBUG_EXTRA=''
 WITH_SSL='/usr'
 WITH_SSL_TYPE='system'
 OPENSSL_INCLUDE=''
@@ -28,7 +28,7 @@ TAG=''
 CMAKE_BUILD_TYPE=''
 COMMON_FLAGS=''
 #
-TOKUDB_BACKUP_VERSION='5.6.28-76.1'
+TOKUDB_BACKUP_VERSION='5.6.33-79.0'
 #
 # Some programs that may be overriden
 TAR=${TAR:-tar}
@@ -56,7 +56,7 @@ do
         shift
         CMAKE_BUILD_TYPE='Debug'
         BUILD_COMMENT="${BUILD_COMMENT:-}-debug"
-        DEBUG_EXTNAME='-DDEBUG_EXTNAME=OFF'
+        DEBUG_EXTRA="-DDEBUG_EXTNAME=OFF"
         ;;
     -v | --valgrind )
         shift
@@ -172,6 +172,15 @@ COMMENT="$COMMENT, Revision $REVISION${BUILD_COMMENT:-}"
 export CC=${CC:-gcc}
 export CXX=${CXX:-g++}
 
+# If gcc >= 4.8 we can use ASAN in debug build but not if valgrind build also
+if [[ "$CMAKE_BUILD_TYPE" == "Debug" ]] && [[ "${CMAKE_OPTS:-}" != *WITH_VALGRIND=ON* ]]; then
+  GCC_VERSION=$(${CC} -dumpversion)
+  GT_VERSION=$(echo -e "4.8.0\n${GCC_VERSION}" | sort -t. -k1,1nr -k2,2nr -k3,3nr | head -1)
+  if [ "${GT_VERSION}" = "${GCC_VERSION}" ]; then
+    DEBUG_EXTRA="${DEBUG_EXTRA} -DWITH_ASAN=ON"
+  fi
+fi
+
 # TokuDB cmake flags
 if test -d "$SOURCEDIR/storage/tokudb"
 then
@@ -235,7 +244,7 @@ fi
  
     cmake . ${CMAKE_OPTS:-} -DBUILD_CONFIG=mysql_release \
         -DCMAKE_BUILD_TYPE=${CMAKE_BUILD_TYPE:-RelWithDebInfo} \
-        $DEBUG_EXTNAME \
+        $DEBUG_EXTRA \
         -DWITH_EMBEDDED_SERVER=OFF \
         -DFEATURE_SET=community \
         -DENABLE_DTRACE=OFF \
