@@ -32,7 +32,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
     along with Percona TokuBackup.  If not, see <http://www.gnu.org/licenses/>.
 ======= */
 
-#ident "$Id: 487fa12c292809b18362a8c98f9de8405588dc77 $"
+#ident "$Id: 04ed43283ccfd8fc9fbe6ed0f0c63e749956d112 $"
 
 #include "backup_debug.h"
 #include "file_hash_table.h"
@@ -54,7 +54,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 #include <string.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <valgrind/helgrind.h>
+#include "backup_helgrind.h"
 
 #if DEBUG_HOTBACKUP
 #define WARN(string, arg) HotBackup::CaptureWarn(string, arg)
@@ -111,13 +111,13 @@ manager::manager(void) throw()
       m_errnum(BACKUP_SUCCESS),
       m_errstring(NULL)
 {
-    VALGRIND_HG_DISABLE_CHECKING(&m_backup_is_running, sizeof(m_backup_is_running));
+    TOKUBACKUP_VALGRIND_HG_DISABLE_CHECKING(&m_backup_is_running, sizeof(m_backup_is_running));
 #ifdef GLASSBOX
-    VALGRIND_HG_DISABLE_CHECKING(&m_is_capturing, sizeof(m_is_capturing));
-    VALGRIND_HG_DISABLE_CHECKING(&m_done_copying, sizeof(m_done_copying));
-    VALGRIND_HG_DISABLE_CHECKING(&m_an_error_happened, sizeof(m_an_error_happened));
-    VALGRIND_HG_DISABLE_CHECKING(&m_errnum, sizeof(m_errnum));
-    VALGRIND_HG_DISABLE_CHECKING(&m_errstring, sizeof(m_errstring));
+    TOKUBACKUP_VALGRIND_HG_DISABLE_CHECKING(&m_is_capturing, sizeof(m_is_capturing));
+    TOKUBACKUP_VALGRIND_HG_DISABLE_CHECKING(&m_done_copying, sizeof(m_done_copying));
+    TOKUBACKUP_VALGRIND_HG_DISABLE_CHECKING(&m_an_error_happened, sizeof(m_an_error_happened));
+    TOKUBACKUP_VALGRIND_HG_DISABLE_CHECKING(&m_errnum, sizeof(m_errnum));
+    TOKUBACKUP_VALGRIND_HG_DISABLE_CHECKING(&m_errstring, sizeof(m_errstring));
 #endif
 }
 
@@ -264,6 +264,10 @@ int manager::prepare_directories_for_backup(backup_session *session, backtrace b
         }
 
         with_object_to_free<char*> file_name(session->translate_prefix_of_realpath(source->name()));
+        if(session->file_is_excluded(file_name.value)) {
+            goto out;
+        }
+
         r = open_path(file_name.value);
         if (r != 0) {
             backup_error(r, "Failed to open path");
@@ -746,7 +750,7 @@ int manager::unlink(const char *path) throw() {
         with_rwlock_rdlocked ms(&m_session_rwlock);
         with_file_hash_table_mutex mtl(&m_table);
 
-        if (m_session != NULL && this->capture_is_enabled() && m_session->is_prefix_of_realpath(full_path.value)) {
+        if (this->should_capture_unlink_of_file(full_path.value)) {
             // 1. Find source file, unlink it.
             // 2. Get destination file from source file.
             destination_file * dest = source->get_destination();
@@ -930,7 +934,7 @@ void manager::mkdir(const char *pathname) throw() {
 ///////////////////////////////////////////////////////////////////////////////
 //
 void manager::set_throttle(unsigned long bytes_per_second) throw() {
-    VALGRIND_HG_DISABLE_CHECKING(&m_throttle, sizeof(m_throttle));
+    TOKUBACKUP_VALGRIND_HG_DISABLE_CHECKING(&m_throttle, sizeof(m_throttle));
     m_throttle = bytes_per_second;
 }
 
@@ -1054,15 +1058,26 @@ void manager::unlock_file_op(void)
     pmutex_unlock(&m_atomic_file_op_mutex);
 }
 
+bool manager::should_capture_unlink_of_file(const char *file) throw() {
+    if (m_session != NULL &&
+        this->capture_is_enabled() &&
+        m_session->is_prefix_of_realpath(file) &&
+        !m_session->file_is_excluded(file)) {
+        return true;
+    }
+
+    return false;
+}
+
 #ifdef GLASSBOX
 // Test routines.
 void manager::pause_disable(bool pause) throw() {
-    VALGRIND_HG_DISABLE_CHECKING(&m_pause_disable, sizeof(m_pause_disable));
+    TOKUBACKUP_VALGRIND_HG_DISABLE_CHECKING(&m_pause_disable, sizeof(m_pause_disable));
     m_pause_disable = pause;
 }
 
 void manager::set_keep_capturing(bool keep_capturing) throw() {
-    VALGRIND_HG_DISABLE_CHECKING(&m_keep_capturing, sizeof(m_keep_capturing));
+    TOKUBACKUP_VALGRIND_HG_DISABLE_CHECKING(&m_keep_capturing, sizeof(m_keep_capturing));
     m_keep_capturing = keep_capturing;
 }
 
@@ -1074,7 +1089,7 @@ bool manager::is_capturing(void) throw() {
 }
 
 void manager::set_start_copying(bool start_copying) throw() {
-    VALGRIND_HG_DISABLE_CHECKING(&m_start_copying, sizeof(m_start_copying));
+    TOKUBACKUP_VALGRIND_HG_DISABLE_CHECKING(&m_start_copying, sizeof(m_start_copying));
     m_start_copying = start_copying;
 }
 #endif /*GLASSBOX*/
