@@ -23,6 +23,7 @@
 #include <sql_audit.h>
 #include <debug_sync.h>
 #include <threadpool.h>
+#include <global_threads.h>
 #include <probes_mysql.h>
 
 
@@ -73,17 +74,16 @@ struct Worker_thread_context
 
   void save()
   {
-#ifdef HAVE_PSI_INTERFACE
-    psi_thread= PSI_server ? PSI_server->get_thread() : 0;
+#ifdef HAVE_PSI_THREAD_INTERFACE
+    psi_thread= PSI_THREAD_CALL(get_thread)();
 #endif
     mysys_var= (st_my_thread_var *)pthread_getspecific(THR_KEY_mysys);
   }
 
   void restore()
   {
-#ifdef HAVE_PSI_INTERFACE
-    if (PSI_server)
-      PSI_server->set_thread(psi_thread);
+#ifdef HAVE_PSI_THREAD_INTERFACE
+    PSI_THREAD_CALL(set_thread)(psi_thread);
 #endif
     pthread_setspecific(THR_KEY_mysys,mysys_var);
     pthread_setspecific(THR_THD, 0);
@@ -100,13 +100,82 @@ static bool thread_attach(THD* thd)
   pthread_setspecific(THR_KEY_mysys,thd->mysys_var);
   thd->thread_stack=(char*)&thd;
   thd->store_globals();
-#ifdef HAVE_PSI_INTERFACE
-  if (PSI_server)
-    PSI_server->set_thread(thd->event_scheduler.m_psi);
+#ifdef HAVE_PSI_THREAD_INTERFACE
+  PSI_THREAD_CALL(set_thread)(thd->event_scheduler.m_psi);
 #endif
+  mysql_socket_set_thread_owner(thd->net.vio->mysql_socket);
   return 0;
 }
 
+#ifdef HAVE_PSI_STATEMENT_INTERFACE
+extern PSI_statement_info stmt_info_new_packet;
+#endif
+
+static void threadpool_net_before_header_psi_noop(struct st_net * /* net */,
+                                                  void * /* user_data */,
+                                                  size_t /* count */)
+{ }
+
+static void threadpool_net_after_header_psi(struct st_net *net,
+                                            void *user_data,
+                                            size_t /* count */, my_bool rc)
+{
+  THD *thd;
+  thd= static_cast<THD*> (user_data);
+  DBUG_ASSERT(thd != NULL);
+
+  if (thd->m_server_idle)
+  {
+    /*
+      The server just got data for a network packet header,
+      from the network layer.
+      The IDLE event is now complete, since we now have a message to process.
+      We need to:
+      - start a new STATEMENT event
+      - start a new STAGE event, within this statement,
+      - start recording SOCKET WAITS events, within this stage.
+      The proper order is critical to get events numbered correctly,
+      and nested in the proper parent.
+    */
+    MYSQL_END_IDLE_WAIT(thd->m_idle_psi);
+
+    if (! rc)
+    {
+      thd->m_statement_psi= MYSQL_START_STATEMENT(&thd->m_statement_state,
+                                                  stmt_info_new_packet.m_key,
+                                                  thd->db, thd->db_length,
+                                                  thd->charset());
+
+      THD_STAGE_INFO(thd, stage_init);
+    }
+
+    /*
+      TODO: consider recording a SOCKET event for the bytes just read,
+      by also passing count here.
+    */
+    MYSQL_SOCKET_SET_STATE(net->vio->mysql_socket, PSI_SOCKET_STATE_ACTIVE);
+
+    thd->m_server_idle = false;
+  }
+}
+
+static void threadpool_init_net_server_extension(THD *thd)
+{
+#ifdef HAVE_PSI_INTERFACE
+  /* Start with a clean state for connection events. */
+  thd->m_idle_psi= NULL;
+  thd->m_statement_psi= NULL;
+  thd->m_server_idle= false;
+  /* Hook up the NET_SERVER callback in the net layer. */
+  thd->m_net_server_extension.m_user_data= thd;
+  thd->m_net_server_extension.m_before_header= threadpool_net_before_header_psi_noop;
+  thd->m_net_server_extension.m_after_header= threadpool_net_after_header_psi;
+  /* Activate this private extension for the mysqld server. */
+  thd->net.extension= & thd->m_net_server_extension;
+#else
+  thd->net.extension= NULL;
+#endif
+}
 
 int threadpool_add_connection(THD *thd)
 {
@@ -130,12 +199,9 @@ int threadpool_add_connection(THD *thd)
   }
 
   /* Create new PSI thread for use with the THD. */
-#ifdef HAVE_PSI_INTERFACE
-  if (PSI_server)
-  {
-    thd->event_scheduler.m_psi = 
-      PSI_server->new_thread(key_thread_one_connection, thd, thd->thread_id);
-  }
+#ifdef HAVE_PSI_THREAD_INTERFACE
+  thd->event_scheduler.m_psi=
+    PSI_THREAD_CALL(new_thread)(key_thread_one_connection, thd, thd->thread_id);
 #endif
 
 
@@ -163,6 +229,9 @@ int threadpool_add_connection(THD *thd)
         retval= 0;
         thd->net.reading_or_writing= 1;
         thd->skip_wait_timeout= true;
+        MYSQL_SOCKET_SET_STATE(thd->net.vio->mysql_socket, PSI_SOCKET_STATE_IDLE);
+        thd->m_server_idle= true;
+        threadpool_init_net_server_extension(thd);
       }
 
       MYSQL_CONNECTION_START(thd->thread_id, &thd->security_ctx->priv_user[0],
@@ -170,6 +239,7 @@ int threadpool_add_connection(THD *thd)
     }
   }
   worker_context.restore();
+  thd->system_tid = -1;
   return retval;
 }
 
@@ -186,10 +256,12 @@ void threadpool_remove_connection(THD *thd)
   end_connection(thd);
   close_connection(thd, 0);
 
-  unlink_thd(thd);
-  mysql_mutex_unlock(&LOCK_thd_remove);
-  mysql_mutex_unlock(&LOCK_thread_count);
-  mysql_cond_broadcast(&COND_thread_count);
+  thd->release_resources();
+
+  remove_global_thread(thd);
+  dec_connection_count(thd);
+
+  delete thd;
 
   /*
     Free resources associated with this connection: 
@@ -197,6 +269,8 @@ void threadpool_remove_connection(THD *thd)
   */
   my_thread_end();
 
+  mysql_cond_broadcast(&COND_thread_count);
+
   worker_context.restore();
 }
 
@@ -254,10 +328,22 @@ int threadpool_process_request(THD *thd)
       thd->net.reading_or_writing= 1;
       goto end;
     }
+    if (!thd->m_server_idle) {
+      MYSQL_SOCKET_SET_STATE(thd->net.vio->mysql_socket, PSI_SOCKET_STATE_IDLE);
+      MYSQL_START_IDLE_WAIT(thd->m_idle_psi, &thd->m_idle_state);
+      thd->m_server_idle= true;
+    }
   }
 
 end:
+  if (!retval && !thd->m_server_idle) {
+    MYSQL_SOCKET_SET_STATE(thd->net.vio->mysql_socket, PSI_SOCKET_STATE_IDLE);
+    MYSQL_START_IDLE_WAIT(thd->m_idle_psi, &thd->m_idle_state);
+    thd->m_server_idle= true;
+  }
+
   worker_context.restore();
+  thd->system_tid = -1;
   return retval;
 }
 
