@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2015, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1995, 2016, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -67,12 +67,6 @@ UNIV_INTERN mysql_pfs_key_t buf_page_cleaner_thread_key;
 UNIV_INTERN mysql_pfs_key_t buf_lru_manager_thread_key;
 #endif /* UNIV_PFS_THREAD */
 
-/** If LRU list of a buf_pool is less than this size then LRU eviction
-should not happen. This is because when we do LRU flushing we also put
-the blocks on free list. If LRU list is very small then we can end up
-in thrashing. */
-#define BUF_LRU_MIN_LEN		256
-
 /* @} */
 
 /** Handled page counters for a single flush */
@@ -315,6 +309,8 @@ buf_flush_init_flush_rbt(void)
 
 		buf_flush_list_mutex_enter(buf_pool);
 
+		ut_ad(buf_pool->flush_rbt == NULL);
+
 		/* Create red black tree for speedy insertions in flush list. */
 		buf_pool->flush_rbt = rbt_create(
 			sizeof(buf_page_t*), buf_flush_block_cmp);
@@ -1494,7 +1490,7 @@ It attempts to make 'max' blocks available in the free list. Note that
 it is a best effort attempt and it is not guaranteed that after a call
 to this function there will be 'max' blocks in the free list.
 @return number of blocks for which the write request was queued. */
-__attribute__((nonnull))
+MY_ATTRIBUTE((nonnull))
 static
 void
 buf_flush_LRU_list_batch(
@@ -1640,7 +1636,7 @@ Whether LRU or unzip_LRU is used depends on the state of the system.
 @return number of blocks for which either the write request was queued
 or in case of unzip_LRU the number of blocks actually moved to the
 free list */
-__attribute__((nonnull))
+MY_ATTRIBUTE((nonnull))
 static
 void
 buf_do_LRU_batch(
@@ -1760,7 +1756,7 @@ pages: to avoid deadlocks, this function must be written so that it cannot
 end up waiting for these latches! NOTE 2: in the case of a flush list flush,
 the calling thread is not allowed to own any latches on pages!
 @return number of blocks for which the write request was queued */
-__attribute__((nonnull))
+MY_ATTRIBUTE((nonnull))
 static
 void
 buf_flush_batch(
@@ -1938,7 +1934,7 @@ list.
 NOTE: The calling thread is not allowed to own any latches on pages!
 @return true if a batch was queued successfully. false if another batch
 of same type was already running. */
-__attribute__((nonnull))
+MY_ATTRIBUTE((nonnull))
 static
 bool
 buf_flush_LRU(
@@ -2572,6 +2568,11 @@ page_cleaner_sleep_if_needed(
 	ulint	next_loop_time)	/*!< in: time when next loop iteration
 				should start */
 {
+	/* No sleep if we are cleaning the buffer pool during the shutdown
+	with everything else finished */
+	if (srv_shutdown_state == SRV_SHUTDOWN_FLUSH_PHASE)
+		return;
+
 	ulint	cur_time = ut_time_ms();
 
 	if (next_loop_time > cur_time) {
@@ -2587,7 +2588,7 @@ page_cleaner_sleep_if_needed(
 /*********************************************************************//**
 Returns the aggregate free list length over all buffer pool instances.
 @return total free list length. */
-__attribute__((warn_unused_result))
+MY_ATTRIBUTE((warn_unused_result))
 static
 ulint
 buf_get_total_free_list_length(void)
@@ -2605,7 +2606,7 @@ buf_get_total_free_list_length(void)
 
 /*********************************************************************//**
 Adjust the desired page cleaner thread sleep time for LRU flushes.  */
-__attribute__((nonnull))
+MY_ATTRIBUTE((nonnull))
 static
 void
 page_cleaner_adapt_lru_sleep_time(
@@ -2644,7 +2645,7 @@ page_cleaner_adapt_lru_sleep_time(
 /*********************************************************************//**
 Get the desired page cleaner thread sleep time for flush list flushes.
 @return desired sleep time */
-__attribute__((warn_unused_result))
+MY_ATTRIBUTE((warn_unused_result))
 static
 ulint
 page_cleaner_adapt_flush_sleep_time(void)
@@ -2671,7 +2672,7 @@ extern "C" UNIV_INTERN
 os_thread_ret_t
 DECLARE_THREAD(buf_flush_page_cleaner_thread)(
 /*==========================================*/
-	void*	arg __attribute__((unused)))
+	void*	arg MY_ATTRIBUTE((unused)))
 			/*!< in: a dummy parameter required by
 			os_thread_create */
 {
@@ -2827,7 +2828,7 @@ extern "C" UNIV_INTERN
 os_thread_ret_t
 DECLARE_THREAD(buf_flush_lru_manager_thread)(
 /*==========================================*/
-	void*	arg __attribute__((unused)))
+	void*	arg MY_ATTRIBUTE((unused)))
 			/*!< in: a dummy parameter required by
 			os_thread_create */
 {
